@@ -32,7 +32,5 @@ static const struct fdt_match serial_sifive_match[] = {
 
 struct fdt_serial fdt_serial_sifive = {
 	.match_table = serial_sifive_match,
-	.init = serial_sifive_init,
-	.getc = sifive_uart_getc,
-	.putc = sifive_uart_putc
+	.init = serial_sifive_init
 };
